@@ -26,7 +26,6 @@
 #include <gtk/gtk.h>
 #include <gdk/gdkx.h>
 #include <gdk/gdkkeysyms.h>
-#include <glade/glade.h>
 
 #include <limits.h>
 #include <sys/param.h>
@@ -83,20 +82,18 @@ static char *hostname_titles[] = {
     N_("Calculator [%s] - Programming")
 };
 
-#define FINC_NUM_ARGS 4
-
 /* The names of each field in the dialogs for the financial functions */
-static char *finc_dialog_fields[FINC_NUM_DIALOGS][FINC_NUM_ARGS] = {
-    {"ctrm_pint", "ctrm_fv",     "ctrm_pv",    NULL},
-    {"ddb_cost",  "ddb_life",    "ddb_period", NULL},
-    {"fv_pmt",    "fv_pint",     "fv_n",       NULL},
-    {"gpm_cost",  "gpm_margin",  NULL,         NULL},
-    {"pmt_prin",  "pmt_pint",    "pmt_n",      NULL},
-    {"pv_pmt",    "pv_pint",     "pv_n",       NULL},
-    {"rate_fv",   "rate_pv",     "rate_n",     NULL},
-    {"sln_cost",  "sln_salvage", "sln_life",   NULL},
-    {"syd_cost",  "syd_salvage", "syd_life",   "syd_period"},
-    {"term_pmt",  "term_fv",     "term_pint",  NULL},
+static char *finc_dialog_fields[FINC_NUM_DIALOGS][5] = {
+    {"ctrm_pint", "ctrm_fv",     "ctrm_pv",    NULL,         NULL},
+    {"ddb_cost",  "ddb_life",    "ddb_period", NULL,         NULL},
+    {"fv_pmt",    "fv_pint",     "fv_n",       NULL,         NULL},
+    {"gpm_cost",  "gpm_margin",  NULL,         NULL,         NULL},
+    {"pmt_prin",  "pmt_pint",    "pmt_n",      NULL,         NULL},
+    {"pv_pmt",    "pv_pint",     "pv_n",       NULL,         NULL},
+    {"rate_fv",   "rate_pv",     "rate_n",     NULL,         NULL},
+    {"sln_cost",  "sln_salvage", "sln_life",   NULL,         NULL},
+    {"syd_cost",  "syd_salvage", "syd_life",   "syd_period", NULL},
+    {"term_pmt",  "term_fv",     "term_pint",  NULL,         NULL},
 };
 
 /*  This table shows the keyboard values that are currently being used:
@@ -442,15 +439,14 @@ static struct button_widget button_widgets[] = {
 };
 #define NBUTTONS (sizeof(button_widgets) / sizeof(struct button_widget))
 
-#define UI_FILE PACKAGE_GLADE_DIR "/gcalctool.glade"
+#define UI_FILE PACKAGE_UI_DIR "/gcalctool.ui"
 
-#define  MAXBITS    64      /* Bit panel: number of bit fields. */
+#define MAXBITS 64      /* Bit panel: number of bit fields. */
 
 #define GET_WIDGET(name) \
-          glade_xml_get_widget(X.ui, (name))
-
-#define CONNECT_SIGNAL(name) glade_xml_signal_connect(X.ui, #name, \
-                       G_CALLBACK(name))
+          GTK_WIDGET(gtk_builder_get_object(X.ui, (name)))
+#define GET_FINC_WIDGET(name) \
+          GTK_WIDGET(gtk_builder_get_object(X.financial, (name)))
 
 /* Gtk+/Xlib graphics object. */
 typedef struct {
@@ -459,8 +455,8 @@ typedef struct {
    
     ModeType mode;  /* Current calculator mode. */   
 
-    GladeXML  *ui;
-    GladeXML  *financial;
+    GtkBuilder *ui;
+    GtkBuilder *financial;
     
     GtkWidget *kframe;                 /* Main window. */
  
@@ -879,8 +875,8 @@ do_finc(char* dialog)
     if (X.financial == NULL) {
         setup_finc_dialogs();
     }
-    gtk_dialog_run(GTK_DIALOG(glade_xml_get_widget(X.financial, dialog)));
-    gtk_widget_hide(GTK_WIDGET(glade_xml_get_widget(X.financial, dialog)));
+    gtk_dialog_run(GTK_DIALOG(GET_FINC_WIDGET(dialog)));
+    gtk_widget_hide(GTK_WIDGET(GET_FINC_WIDGET(dialog)));
 }
 
 void
@@ -894,16 +890,11 @@ ui_set_mode(ModeType mode)
     if (X.mode != mode) {
         X.mode = mode;
 
-        // FIXME: These should affect display but not the actual UI settings
-        if (mode != PROGRAMMING)
-            ui_set_base(DEC);
-        if (mode != SCIENTIFIC) {
-            ui_set_numeric_mode(FIX);
-            do_button(FN_SET_ACCURACY, DEFAULT_ACCURACY);
-        }
-        if (mode == BASIC)
-            ui_set_show_trailing_zeroes(FALSE);
-
+        ui_set_base(DEC);
+        ui_set_numeric_mode(FIX);
+        do_button(FN_SET_ACCURACY, DEFAULT_ACCURACY);
+        ui_set_show_thousands_separator(FALSE);
+        ui_set_show_trailing_zeroes(FALSE);
         ui_make_registers();
     }
     
@@ -1187,8 +1178,8 @@ ui_set_registers_visible(gboolean visible)
 }
 
 
-/*ARGSUSED*/
-static void
+G_MODULE_EXPORT
+void
 about_cb(GtkWidget *widget)
 {
     const gchar *authors[] = {
@@ -1206,8 +1197,8 @@ about_cb(GtkWidget *widget)
     /* Translators: The translator credits. Please translate this with your name(s). */
     const gchar *translator_credits = _("translator-credits");
 
-    /* Translators: The license this software is under (GPL2+) */
-    char *license =
+    char *license[] = {
+        /* Translators: The license this software is under (GPL2+) */
         _("Gcalctool is free software; you can redistribute it and/or modify\n"
           "it under the terms of the GNU General Public License as published by\n"
           "the Free Software Foundation; either version 2 of the License, or\n"
@@ -1220,7 +1211,9 @@ about_cb(GtkWidget *widget)
           "\n"
           "You should have received a copy of the GNU General Public License\n"
           "along with Gcalctool; if not, write to the Free Software Foundation, Inc.,\n"
-          "51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA\n");
+          "51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA\n"),
+        NULL
+    };
 
     gtk_show_about_dialog(GTK_WINDOW(X.kframe),
                           /* Translators: Program name in the about dialog */
@@ -1295,24 +1288,23 @@ add_cf_column(GtkTreeView *treeview, gchar *name, gint colno, gboolean editable)
 }
 
 
-/*ARGSUSED*/
-static void
+G_MODULE_EXPORT
+void
 aframe_response_cb(GtkWidget *dialog, gint response_id)
 {
-    char *ch;
-
-    if (response_id == GTK_RESPONSE_OK) {
-        int value[MP_SIZE];
-        ch = (char *) gtk_entry_get_text(GTK_ENTRY(X.aframe_ch));
-        mp_set_from_integer(ch[0], value);
-        display_set_number(&v->display, value);
-    }
+    const gchar *text;
+    
+    text = gtk_entry_get_text(GTK_ENTRY(X.aframe_ch));
+    
+    if (response_id == GTK_RESPONSE_OK)
+        do_button(FN_INSERT_CHARACTER, GPOINTER_TO_INT(text));
     
     gtk_widget_hide(dialog);
 }
 
 
-static gboolean
+G_MODULE_EXPORT
+gboolean
 aframe_delete_cb(GtkWidget *dialog)
 {
     aframe_response_cb(dialog, GTK_RESPONSE_CANCEL);
@@ -1320,23 +1312,24 @@ aframe_delete_cb(GtkWidget *dialog)
 }
 
 
-/*ARGSUSED*/
-static void
+G_MODULE_EXPORT
+void
 aframe_activate_cb(GtkWidget *entry)
 {
     aframe_response_cb(X.aframe, GTK_RESPONSE_OK);
 }
 
 
-/*ARGSUSED*/
-static void
+G_MODULE_EXPORT
+void
 rframe_response_cb(GtkWidget *dialog, int response_id)
 {
     ui_set_registers_visible(FALSE);
 }
 
 
-static gboolean
+G_MODULE_EXPORT
+gboolean
 rframe_delete_cb(GtkWidget *dialog)
 {
     rframe_response_cb(dialog, GTK_RESPONSE_OK);
@@ -1344,7 +1337,7 @@ rframe_delete_cb(GtkWidget *dialog)
 }
 
 
-/*ARGSUSED*/
+G_MODULE_EXPORT
 void
 disp_cb(GtkWidget *widget)
 {
@@ -1353,7 +1346,7 @@ disp_cb(GtkWidget *widget)
 }
 
 
-/*ARGSUSED*/
+G_MODULE_EXPORT
 void
 base_cb(GtkWidget *widget)
 {
@@ -1373,7 +1366,7 @@ help_display(void)
     GError *error = NULL;
 
     screen = gtk_widget_get_screen (GTK_WIDGET (X.kframe));
-    gtk_show_uri (screen, "ghelp:gcalctool", gtk_get_current_event_time (), &error);
+   //gtk_show_uri (screen, "ghelp:gcalctool", gtk_get_current_event_time (), &error);
  
     if (error != NULL)
     {
@@ -1392,8 +1385,8 @@ help_display(void)
 }
 
 
-/*ARGSUSED*/
-static void
+G_MODULE_EXPORT
+void
 constant_menu_cb(GtkMenuItem *menu)
 {
     int arg = GPOINTER_TO_INT(g_object_get_data(G_OBJECT(menu), "constant_id"));
@@ -1401,8 +1394,8 @@ constant_menu_cb(GtkMenuItem *menu)
 }
 
 
-/*ARGSUSED*/
-static void
+G_MODULE_EXPORT
+void
 function_menu_cb(GtkMenuItem *menu)
 {
     int arg = GPOINTER_TO_INT(g_object_get_data(G_OBJECT(menu), "function_id"));
@@ -1410,8 +1403,8 @@ function_menu_cb(GtkMenuItem *menu)
 }
 
 
-/*ARGSUSED*/
-static void
+G_MODULE_EXPORT
+void
 store_menu_cb(GtkMenuItem *menu)
 {
     int arg = GPOINTER_TO_INT(g_object_get_data(G_OBJECT(menu), "register_id"));
@@ -1419,8 +1412,8 @@ store_menu_cb(GtkMenuItem *menu)
 }
 
 
-/*ARGSUSED*/
-static void
+G_MODULE_EXPORT
+void
 recall_menu_cb(GtkMenuItem *menu)
 {
     int arg = GPOINTER_TO_INT(g_object_get_data(G_OBJECT(menu), "register_id"));
@@ -1428,8 +1421,8 @@ recall_menu_cb(GtkMenuItem *menu)
 }
 
 
-/*ARGSUSED*/
-static void
+G_MODULE_EXPORT
+void
 exchange_menu_cb(GtkMenuItem *menu)
 {
     int arg = GPOINTER_TO_INT(g_object_get_data(G_OBJECT(menu), "register_id"));
@@ -1437,104 +1430,108 @@ exchange_menu_cb(GtkMenuItem *menu)
 }
 
 
-static void
-finc_activate_cb(GtkWidget *widget, void *pointer) {
-    gint i, dialog;
-    GtkWidget *next_widget, *dialog_widget;
-    const char* widget_name = glade_get_widget_name(widget);
-    for (dialog = 0; dialog < FINC_NUM_DIALOGS; dialog++) {
-        for (i = 0; i < FINC_NUM_ARGS; i++) {
-            if (finc_dialog_fields[dialog][i] == NULL ||
-                g_ascii_strcasecmp(widget_name, 
-                                   finc_dialog_fields[dialog][i])) {
-                continue;
-            }
-            
-            if (i < FINC_NUM_ARGS - 1 &&
-                finc_dialog_fields[dialog][i+1] != NULL) {
-                next_widget = glade_xml_get_widget(X.financial,
-                                                   finc_dialog_fields[dialog][i+1]);
-                gtk_widget_grab_focus(next_widget);
-                return;
-            }
-            else {
-                dialog_widget = gtk_widget_get_toplevel(widget);
-                if (GTK_WIDGET_TOPLEVEL (dialog_widget)) {
-                    gtk_dialog_response(GTK_DIALOG(dialog_widget),
-                                        GTK_RESPONSE_OK);
-                    return;
-                }
-            }
+G_MODULE_EXPORT
+void
+finc_activate_cb(GtkWidget *widget) {
+    gint dialog, field;
+
+    dialog = GPOINTER_TO_INT(g_object_get_data(G_OBJECT(widget), "finc_dialog"));
+    field = GPOINTER_TO_INT(g_object_get_data(G_OBJECT(widget), "finc_field"));
+    
+    if (finc_dialog_fields[dialog][field+1] == NULL) {
+        GtkWidget *dialog_widget;
+        dialog_widget = gtk_widget_get_toplevel(widget);
+        if (GTK_WIDGET_TOPLEVEL (dialog_widget)) {
+            gtk_dialog_response(GTK_DIALOG(dialog_widget),
+                                GTK_RESPONSE_OK);
+            return;
         }
     }
+    else {
+        GtkWidget *next_widget;
+        next_widget = GET_FINC_WIDGET(finc_dialog_fields[dialog][field+1]);
+        gtk_widget_grab_focus(next_widget);
+    }
 }
 
 
-static void
-finc_response_cb(GtkWidget *widget, gint response_id, void *dialog_pointer)
+G_MODULE_EXPORT
+void
+finc_response_cb(GtkWidget *widget, gint response_id)
 {
-    int dialog = GPOINTER_TO_INT (dialog_pointer);
+    int dialog;
     int i;
-    int arg[FINC_NUM_ARGS][MP_SIZE];
+    MPNumber arg[4];
     GtkWidget *entry;
+
     if (response_id != GTK_RESPONSE_OK) {
         return;
     }
+    
+    dialog = GPOINTER_TO_INT (g_object_get_data(G_OBJECT(widget), "finc_dialog"));
 
-    for (i = 0; i < FINC_NUM_ARGS; i++) {
+    for (i = 0; i < 4; i++) {
         if (finc_dialog_fields[dialog][i] == NULL) {
             continue;
         }
-        entry = glade_xml_get_widget(X.financial,
-                                     finc_dialog_fields[dialog][i]);
-        mp_set_from_string(gtk_entry_get_text(GTK_ENTRY(entry)), 10, arg[i]);
+        entry = GET_FINC_WIDGET(finc_dialog_fields[dialog][i]);
+        mp_set_from_string(gtk_entry_get_text(GTK_ENTRY(entry)), 10, &arg[i]);
         gtk_entry_set_text(GTK_ENTRY(entry), "0");
     }
-    gtk_widget_grab_focus(glade_xml_get_widget(X.financial, 
-                                               finc_dialog_fields[dialog][0]));
+    gtk_widget_grab_focus(GET_FINC_WIDGET(finc_dialog_fields[dialog][0]));
 
-    do_finc_expression(dialog, arg[0], arg[1], arg[2], arg[3]);
+    do_finc_expression(dialog, &arg[0], &arg[1], &arg[2], &arg[3]);
 }
 
 
 static void
 setup_finc_dialogs(void)
 {
-    X.financial = glade_xml_new(PACKAGE_GLADE_DIR "/financial.glade", NULL, 
-                                 NULL);
-    glade_xml_signal_connect_data(X.financial, "finc_ctrm_response_cb", 
-                             G_CALLBACK(finc_response_cb), 
-                             GINT_TO_POINTER(FINC_CTRM_DIALOG));
-    glade_xml_signal_connect_data(X.financial, "finc_ddb_response_cb", 
-                             G_CALLBACK(finc_response_cb), 
-                             GINT_TO_POINTER(FINC_DDB_DIALOG));
-    glade_xml_signal_connect_data(X.financial, "finc_fv_response_cb", 
-                             G_CALLBACK(finc_response_cb), 
-                             GINT_TO_POINTER(FINC_FV_DIALOG));
-    glade_xml_signal_connect_data(X.financial, "finc_gpm_response_cb", 
-                             G_CALLBACK(finc_response_cb), 
-                             GINT_TO_POINTER(FINC_GPM_DIALOG));
-    glade_xml_signal_connect_data(X.financial, "finc_pmt_response_cb", 
-                             G_CALLBACK(finc_response_cb), 
-                             GINT_TO_POINTER(FINC_PMT_DIALOG));
-    glade_xml_signal_connect_data(X.financial, "finc_pv_response_cb", 
-                             G_CALLBACK(finc_response_cb), 
-                             GINT_TO_POINTER(FINC_PV_DIALOG));
-    glade_xml_signal_connect_data(X.financial, "finc_rate_response_cb", 
-                             G_CALLBACK(finc_response_cb), 
-                             GINT_TO_POINTER(FINC_RATE_DIALOG));
-    glade_xml_signal_connect_data(X.financial, "finc_sln_response_cb", 
-                             G_CALLBACK(finc_response_cb), 
-                             GINT_TO_POINTER(FINC_SLN_DIALOG));
-    glade_xml_signal_connect_data(X.financial, "finc_syd_response_cb", 
-                             G_CALLBACK(finc_response_cb), 
-                             GINT_TO_POINTER(FINC_SYD_DIALOG));
-    glade_xml_signal_connect_data(X.financial, "finc_term_response_cb", 
-                             G_CALLBACK(finc_response_cb), 
-                             GINT_TO_POINTER(FINC_TERM_DIALOG));
-
-    glade_xml_signal_connect(X.financial, "finc_activate_cb", 
-                             G_CALLBACK(finc_activate_cb));
+    GError *error = NULL;
+    int i, j;
+    
+    // FIXME: Use same code as main UI
+    X.financial = gtk_builder_new();
+    gtk_builder_add_from_file(X.financial, PACKAGE_UI_DIR "/financial.ui", &error);
+    if (error != NULL)
+    {
+        g_object_unref(X.financial);
+        X.financial = NULL;
+        g_warning("Error loading financial UI: %s\n", error->message);
+        return;
+    }
+    
+    g_object_set_data(gtk_builder_get_object(X.financial, "ctrm_dialog"),
+                      "finc_dialog", GINT_TO_POINTER(FINC_CTRM_DIALOG));
+    g_object_set_data(gtk_builder_get_object(X.financial, "ddb_dialog"),
+                      "finc_dialog", GINT_TO_POINTER(FINC_DDB_DIALOG));
+    g_object_set_data(gtk_builder_get_object(X.financial, "fv_dialog"),
+                      "finc_dialog", GINT_TO_POINTER(FINC_FV_DIALOG));
+    g_object_set_data(gtk_builder_get_object(X.financial, "gpm_dialog"),
+                      "finc_dialog", GINT_TO_POINTER(FINC_GPM_DIALOG));
+    g_object_set_data(gtk_builder_get_object(X.financial, "pmt_dialog"),
+                      "finc_dialog", GINT_TO_POINTER(FINC_PMT_DIALOG));
+    g_object_set_data(gtk_builder_get_object(X.financial, "pv_dialog"),
+                      "finc_dialog", GINT_TO_POINTER(FINC_PV_DIALOG));
+    g_object_set_data(gtk_builder_get_object(X.financial, "rate_dialog"),
+                      "finc_dialog", GINT_TO_POINTER(FINC_RATE_DIALOG));
+    g_object_set_data(gtk_builder_get_object(X.financial, "sln_dialog"),
+                      "finc_dialog", GINT_TO_POINTER(FINC_SLN_DIALOG));
+    g_object_set_data(gtk_builder_get_object(X.financial, "syd_dialog"),
+                      "finc_dialog", GINT_TO_POINTER(FINC_SYD_DIALOG));
+    g_object_set_data(gtk_builder_get_object(X.financial, "term_dialog"),
+                      "finc_dialog", GINT_TO_POINTER(FINC_TERM_DIALOG));
+    
+    for (i = 0; i < FINC_NUM_DIALOGS; i++) {
+        for (j = 0; finc_dialog_fields[i][j]; j++) {
+            GObject *o;
+            o = gtk_builder_get_object(X.financial, finc_dialog_fields[i][j]);
+            g_object_set_data(o, "finc_field", GINT_TO_POINTER(j));
+            g_object_set_data(o, "finc_dialog", GINT_TO_POINTER(i));
+        }
+    }
+    
+    gtk_builder_connect_signals(X.financial, NULL);
 }
 
 static void
@@ -1579,7 +1576,8 @@ update_functions_menu(void)
 }
 
 
-static void
+G_MODULE_EXPORT
+void
 edit_constants_response_cb(GtkDialog *dialog, gint id)
 {
     GtkTreeIter iter;
@@ -1594,13 +1592,13 @@ edit_constants_response_cb(GtkDialog *dialog, gint id)
     if (id == GTK_RESPONSE_ACCEPT) {
         if (gtk_tree_model_get_iter_first(X.constants_model, &iter)) {
             do {
-                int temp[MP_SIZE];
+                MPNumber temp;
                 gtk_tree_model_get(X.constants_model, &iter,
                                    COLUMN_NUMBER, &number,
                                    COLUMN_VALUE, &value,
                                    COLUMN_DESCRIPTION, &description, -1);
-                mp_set_from_string(value, 10, temp);
-                constant_set(number, description, temp);
+                mp_set_from_string(value, 10, &temp);
+                constant_set(number, description, &temp);
             } while (gtk_tree_model_iter_next(X.constants_model, &iter));
         }
     }
@@ -1609,7 +1607,8 @@ edit_constants_response_cb(GtkDialog *dialog, gint id)
 }
 
 
-static gboolean
+G_MODULE_EXPORT
+gboolean
 edit_constants_delete_cb(GtkDialog *dialog)
 {
     edit_constants_response_cb(dialog, GTK_RESPONSE_CANCEL);
@@ -1617,7 +1616,8 @@ edit_constants_delete_cb(GtkDialog *dialog)
 }
 
 
-static void
+G_MODULE_EXPORT
+void
 edit_functions_response_cb(GtkDialog *dialog, gint id)
 {
     GtkTreeIter iter;
@@ -1645,7 +1645,8 @@ edit_functions_response_cb(GtkDialog *dialog, gint id)
 }
 
 
-static gboolean
+G_MODULE_EXPORT
+gboolean
 edit_functions_delete_cb(GtkDialog *dialog)
 {
     edit_functions_response_cb(dialog, GTK_RESPONSE_CANCEL);
@@ -1653,7 +1654,6 @@ edit_functions_delete_cb(GtkDialog *dialog)
 }
 
 
-/*ARGSUSED*/
 static GtkTreeModel *
 create_constants_model()
 {
@@ -1680,7 +1680,6 @@ create_constants_model()
 }
 
 
-/*ARGSUSED*/
 static GtkTreeModel *
 create_functions_model()
 {
@@ -1712,15 +1711,15 @@ ui_make_registers()            /* Calculate memory register frame values. */
     int n;
 
     for (n = 0; n < MAX_REGISTERS; n++) {
-        int temp[MP_SIZE];
+        MPNumber temp;
         
-        register_get(n, temp);
-        display_make_number(&v->display, mval, MAXLINE, temp, v->base, TRUE);
+        register_get(n, &temp);
+        display_make_number(&v->display, mval, MAXLINE, &temp, v->base, TRUE);
         gtk_entry_set_width_chars(GTK_ENTRY(X.regs[n]), strlen(mval));
         gtk_entry_set_text(GTK_ENTRY(X.regs[n]), mval);
 
         SNPRINTF(key, MAXLINE, "register%d", n);
-        display_make_number(&v->display, value, MAXLINE, temp, DEC, TRUE);
+        display_make_number(&v->display, value, MAXLINE, &temp, DEC, TRUE);
         set_resource(key, value);
     }
 }
@@ -1737,8 +1736,8 @@ save_win_position()
 }
 
 
-/*ARGSUSED*/
-static gboolean
+G_MODULE_EXPORT
+gboolean
 bit_toggle_cb(GtkWidget *event_box, GdkEventButton *event)
 {
     int index;
@@ -1748,7 +1747,8 @@ bit_toggle_cb(GtkWidget *event_box, GdkEventButton *event)
 }
 
 
-static void
+G_MODULE_EXPORT
+void
 menu_item_select_cb(GtkWidget *widget)
 {
     GtkStatusbar *statusbar = GTK_STATUSBAR(X.statusbar);
@@ -1764,8 +1764,8 @@ menu_item_select_cb(GtkWidget *widget)
 }
 
 
-/*ARGSUSED*/
-static void
+G_MODULE_EXPORT
+void
 menu_item_deselect_cb(GtkWidget *widget)
 {
     GtkStatusbar *statusbar = GTK_STATUSBAR(X.statusbar);
@@ -1796,9 +1796,9 @@ update_memory_menus()
     int i;
 
     for (i = 0; i < MAX_REGISTERS; i++) {
-        int temp[MP_SIZE];
-        register_get(i, temp);
-        display_make_number(&v->display, value, MAXLINE, temp, v->base, TRUE);
+        MPNumber temp;
+        register_get(i, &temp);
+        display_make_number(&v->display, value, MAXLINE, &temp, v->base, TRUE);
         SNPRINTF(mstr, MAXLINE, "<span weight=\"bold\">%s_%d:</span>    %s",
         /* Translators: R is the short form of register used inter alia in popup menus */
                 _("R"), i, value);
@@ -1845,15 +1845,15 @@ check_for_localized_numeric_point(int keyval)
 }
 
 
-/*ARGSUSED*/
-static void 
+G_MODULE_EXPORT
+void 
 help_cb(GtkWidget *widget)
 {
     help_display();
 }
 
 
-/*ARGSUSED*/
+G_MODULE_EXPORT
 void
 hyp_cb(GtkWidget *widget)
 {
@@ -1861,7 +1861,7 @@ hyp_cb(GtkWidget *widget)
 }
 
 
-/*ARGSUSED*/
+G_MODULE_EXPORT
 void
 trig_cb(GtkWidget *widget)
 {
@@ -1870,7 +1870,7 @@ trig_cb(GtkWidget *widget)
 }
 
 
-/*ARGSUSED*/
+G_MODULE_EXPORT
 void
 inv_cb(GtkWidget *widget)
 {
@@ -1878,7 +1878,6 @@ inv_cb(GtkWidget *widget)
 }
 
 
-/*ARGSUSED*/
 static void
 menu_pos_func(GtkMenu *menu, gint *x, gint *y,
               gboolean *push_in, gpointer user_data)
@@ -1890,8 +1889,8 @@ menu_pos_func(GtkMenu *menu, gint *x, gint *y,
 }
 
 
-/*ARGSUSED*/
-static void
+G_MODULE_EXPORT
+void
 button_cb(GtkWidget *widget, GdkEventButton *event)
 {
     int function;
@@ -1933,7 +1932,6 @@ button_cb(GtkWidget *widget, GdkEventButton *event)
 }
 
 
-/*ARGSUSED*/
 static void
 select_display_entry(int offset)
 {
@@ -1945,8 +1943,8 @@ select_display_entry(int offset)
 }
 
 
-/*ARGSUSED*/
-static gboolean
+G_MODULE_EXPORT
+gboolean
 kframe_key_press_cb(GtkWidget *widget, GdkEventKey *event)
 {
     int i, j, state;
@@ -2062,7 +2060,8 @@ kframe_key_press_cb(GtkWidget *widget, GdkEventKey *event)
 }
 
 
-static gboolean
+G_MODULE_EXPORT
+gboolean
 kframe_key_release_cb(GtkWidget *widget, GdkEventKey *event)
 {
     if (event->keyval == GDK_Shift_L || event->keyval == GDK_Shift_R) {
@@ -2074,8 +2073,8 @@ kframe_key_release_cb(GtkWidget *widget, GdkEventKey *event)
 }
 
 
-/*ARGSUSED*/
-static void 
+G_MODULE_EXPORT
+void 
 edit_cb(GtkWidget *widget)
 {
     gboolean can_paste, can_copy;
@@ -2089,136 +2088,68 @@ edit_cb(GtkWidget *widget)
 }
 
 
-/*ARGSUSED*/
-static void 
+G_MODULE_EXPORT
+void 
 copy_cb(GtkWidget *widget)
 {
     get_display();
 }
 
 
-/*ARGSUSED*/
 static void
-get_proc(GtkClipboard *clipboard, const gchar *buffer, gpointer data)
+on_paste(GtkClipboard *clipboard, const gchar *text, gpointer data)
 {
-    gchar *dstp, *end_buffer, *srcp, *text, c;
-
-    if (buffer == NULL) {
-        return;
-    }
-
-    end_buffer = (gchar *) (buffer + strlen(buffer));
-    text = malloc(strlen(buffer)+1);
-
-    dstp = text;
-    for (srcp = (gchar *) buffer; srcp < end_buffer; srcp++) {
-        /* If the clipboard buffer contains any occurances of the "thousands
-         * separator", remove them.
-         */
-        if (v->tsep[0] != '\0' && strncmp(srcp, v->tsep, strlen(v->tsep)) == 0) {
-            srcp += strlen(v->tsep) - 1;
-            continue;
-        }
-        
-        /* Replace radix with "." */
-        else if (strncmp(srcp, v->radix, strlen(v->radix)) == 0) {
-            c = '.';
-        }
-
-        /* Replace tabs with spaces */        
-        else if (*srcp == '\t') {
-            c = ' ';
-        }
-        
-        /* Terminate on newlines */        
-        else if (*srcp == '\r' || *srcp == '\n') {
-            c = '\0';
-        }
-        
-        /* If an "A", "B", "C", "D" or "F" character is encountered, it 
-         * will be converted to its lowercase equivalent. If an "E" is 
-         * found,  and the next character is a "-" or a "+", then it 
-         * remains as an upper case "E" (it's assumed to be a possible 
-         * exponential number), otherwise its converted to a lower case 
-         * "e". See bugs #455889 and #469245 for more details.
-         */
-        else if (*srcp >= 'A' && *srcp <= 'F') {
-            if (*srcp == 'E') {
-                c = *srcp;
-                if (srcp < (end_buffer-1)) {
-                    if (*(srcp+1) != '-' &&
-                        *(srcp+1) != '+') {
-                        c = tolower(*srcp);
-                    }
-                }
-            }
-            else
-                c = tolower(*srcp);
-        }
-        
-        else
-            c = *srcp;
-        
-        *dstp++ = c;
-    }
-    *dstp++ = '\0';
-
-    if (display_is_result(&v->display))
-        display_set_string(&v->display, (char *)text, -1);
-    else
-        display_insert(&v->display, get_cursor(), (char *)text);        
-    display_set_cursor(&v->display, -1);
-    free(text);
+    if (text != NULL)
+        do_button(FN_PASTE, GPOINTER_TO_INT(text));
 }
 
 
-/*ARGSUSED*/
-static gboolean
+G_MODULE_EXPORT
+gboolean
 mouse_button_cb(GtkWidget *widget, GdkEventButton *event)
 {
     if (event->button == 2) {
         gtk_clipboard_request_text(gtk_clipboard_get(X.primary_atom),
-                                   get_proc, NULL);
+                                   on_paste, NULL);
     }
 
     return (FALSE);
 }
 
 
-/*ARGSUSED*/
-static void 
+G_MODULE_EXPORT
+void 
 paste_cb(GtkWidget *widget)
 {
     gtk_clipboard_request_text(gtk_clipboard_get(X.clipboard_atom),
-                               get_proc, NULL);
+                               on_paste, NULL);
 }
 
 
-/*ARGSUSED*/
-static void 
+G_MODULE_EXPORT
+void 
 popup_paste_cb(GtkWidget *menu)
 {
     paste_cb(menu);
 }
 
 
-/*ARGSUSED*/
-static void
+G_MODULE_EXPORT
+void
 undo_cb(GtkWidget *widget)
 {
     do_button(FN_UNDO, 0);
 }
 
 
-/*ARGSUSED*/
-static void
+G_MODULE_EXPORT
+void
 redo_cb(GtkWidget *widget)
 {
     do_button(FN_REDO, 0);    
 }
 
 
-/*ARGSUSED*/
 static void
 for_each_menu(GtkWidget *widget, gpointer data)
 {
@@ -2239,27 +2170,28 @@ for_each_menu(GtkWidget *widget, gpointer data)
 }
 
 
-/*ARGSUSED*/
-static void
+G_MODULE_EXPORT
+void
 buffer_populate_popup_cb(GtkTextView *textview, GtkMenu *menu)
 {
     gtk_container_foreach(GTK_CONTAINER(menu), for_each_menu, NULL);
 }
 
 
-/*ARGSUSED*/
-static void
+G_MODULE_EXPORT
+void
 insert_ascii_cb(GtkWidget *widget)
 {
     if (!GTK_WIDGET_VISIBLE(X.aframe)) {
         ds_position_popup(X.kframe, X.aframe, DS_POPUP_LEFT);
     }
-    gtk_window_set_focus(GTK_WINDOW(X.kframe), GTK_WIDGET(X.aframe_ch));
+    gtk_widget_grab_focus(GTK_WIDGET(X.aframe_ch));
     gtk_widget_show(X.aframe);
 }
 
 
-static void
+G_MODULE_EXPORT
+void
 shift_cb(GtkWidget *widget)
 {
     int count = GPOINTER_TO_INT(g_object_get_data(G_OBJECT(widget), 
@@ -2268,8 +2200,8 @@ shift_cb(GtkWidget *widget)
 }
 
 
-/*ARGSUSED*/
-static void
+G_MODULE_EXPORT
+void
 show_registers_cb(GtkWidget *widget)
 {
     gboolean visible;    
@@ -2278,8 +2210,8 @@ show_registers_cb(GtkWidget *widget)
 }
 
 
-/*ARGSUSED*/
-static void
+G_MODULE_EXPORT
+void
 mode_radio_cb(GtkWidget *menu)
 {
     int mode;             /* The new mode. */
@@ -2293,8 +2225,8 @@ mode_radio_cb(GtkWidget *menu)
 }
 
 
-/*ARGSUSED*/
-static void
+G_MODULE_EXPORT
+void
 accuracy_radio_cb(GtkWidget *widget)
 {
     int count;
@@ -2305,28 +2237,28 @@ accuracy_radio_cb(GtkWidget *widget)
 }
 
 
-/*ARGSUSED*/
-static void
+G_MODULE_EXPORT
+void
 accuracy_other_cb(GtkWidget *widget)
 {
     if (!GTK_WIDGET_VISIBLE(X.spframe)) {
         ds_position_popup(X.kframe, X.spframe, DS_POPUP_LEFT);
     }    
-    gtk_window_set_focus(GTK_WINDOW(X.spframe), GTK_WIDGET(X.precision_spin));
+    gtk_widget_grab_focus(GTK_WIDGET(X.precision_spin));
     gtk_widget_show(X.spframe);
 }
 
 
-/*ARGSUSED*/
-static void
+G_MODULE_EXPORT
+void
 accuracy_default_cb(GtkWidget *widget)
 {
     do_button(FN_SET_ACCURACY, DEFAULT_ACCURACY);
 }
 
 
-/*ARGSUSED*/
-static void
+G_MODULE_EXPORT
+void
 show_trailing_zeroes_cb(GtkWidget *widget)
 {
     gboolean visible;    
@@ -2336,8 +2268,8 @@ show_trailing_zeroes_cb(GtkWidget *widget)
 }
 
 
-/*ARGSUSED*/
-static void
+G_MODULE_EXPORT
+void
 quit_cb(GtkWidget *widget)
 {
     save_win_position();
@@ -2345,7 +2277,8 @@ quit_cb(GtkWidget *widget)
 }
 
 
-static void
+G_MODULE_EXPORT
+void
 spframe_response_cb(GtkWidget *dialog, gint response_id)
 {
     int val;
@@ -2358,7 +2291,8 @@ spframe_response_cb(GtkWidget *dialog, gint response_id)
 }
 
 
-static gboolean
+G_MODULE_EXPORT
+gboolean
 spframe_delete_cb(GtkWidget *dialog)
 {
     spframe_response_cb(dialog, GTK_RESPONSE_CANCEL);
@@ -2366,16 +2300,16 @@ spframe_delete_cb(GtkWidget *dialog)
 }
 
 
-/*ARGSUSED*/
-static void
+G_MODULE_EXPORT
+void
 spframe_activate_cb(GtkWidget *spin)
 {
     spframe_response_cb(X.spframe, GTK_RESPONSE_OK);
 }
 
 
-/*ARGSUSED*/
-static void
+G_MODULE_EXPORT
+void
 show_thousands_separator_cb(GtkWidget *widget)
 {
     gboolean visible;
@@ -2385,16 +2319,16 @@ show_thousands_separator_cb(GtkWidget *widget)
 }
 
 
-/*ARGSUSED*/
-static void
+G_MODULE_EXPORT
+void
 edit_constants_cb(GtkMenuItem *item)
 {
     gtk_widget_show(X.con_dialog);
 }
 
 
-/*ARGSUSED*/
-static void
+G_MODULE_EXPORT
+void
 edit_functions_cb(GtkMenuItem *item)
 {
     gtk_widget_show(X.fun_dialog);
@@ -2438,9 +2372,11 @@ create_kframe()
     GtkSizeGroup *size_group;
     GtkAccelGroup *accel_group;
     GtkWidget *treeview;
+    GError *error = NULL;
    
-    X.ui = glade_xml_new(UI_FILE, NULL, NULL);
-    if (X.ui == NULL) {
+    X.ui = gtk_builder_new();
+    gtk_builder_add_from_file(X.ui, UI_FILE, &error);
+    if (error != NULL) {
         GtkWidget *dialog;
         
         dialog = gtk_message_dialog_new(NULL, 0,
@@ -2448,6 +2384,8 @@ create_kframe()
                                         GTK_BUTTONS_NONE,
                                         /* Translators: Title of the error dialog when unable to load the UI files */
                                         N_("Error loading user interface"));
+        // FIXME: Use error->message
+        g_warning("Unable to load file %s: %s\n", UI_FILE, error->message);
         gtk_message_dialog_format_secondary_text(GTK_MESSAGE_DIALOG(dialog),
                                                  /* Translators: Description in UI error dialog when unable to load the UI files. %s is replaced with the path of the missing file */
                                                  N_("The user interface file %s is missing or unable to be loaded. Please check your installation."), UI_FILE);
@@ -2456,65 +2394,7 @@ create_kframe()
         gtk_dialog_run(GTK_DIALOG(dialog));
         exit(0);
     }
-    
-    /* When connecting up signals, would ideally use autoconnect but not 
-     * sure how to get the build process working. 
-     * See http://library.gnome.org/devel/libglade/unstable and
-     * http://www.jamesh.id.au/software/libglade/ 
-     * for some information on how to get this to work
-     * glade_xml_signal_autoconnect(X.ui);
-     */
-    CONNECT_SIGNAL(kframe_key_press_cb);
-    CONNECT_SIGNAL(kframe_key_release_cb);
-    CONNECT_SIGNAL(button_cb);
-    CONNECT_SIGNAL(menu_item_select_cb);
-    CONNECT_SIGNAL(menu_item_deselect_cb);
-    CONNECT_SIGNAL(mode_radio_cb);
-    CONNECT_SIGNAL(inv_cb);
-    CONNECT_SIGNAL(hyp_cb);
-    CONNECT_SIGNAL(base_cb);
-    CONNECT_SIGNAL(disp_cb);
-    CONNECT_SIGNAL(trig_cb);    
-    CONNECT_SIGNAL(quit_cb);
-    CONNECT_SIGNAL(edit_cb);
-    CONNECT_SIGNAL(copy_cb);
-    CONNECT_SIGNAL(paste_cb);
-    CONNECT_SIGNAL(insert_ascii_cb);
-    CONNECT_SIGNAL(undo_cb);
-    CONNECT_SIGNAL(redo_cb);
-    CONNECT_SIGNAL(help_cb);
-    CONNECT_SIGNAL(about_cb);
-    CONNECT_SIGNAL(show_trailing_zeroes_cb);
-    CONNECT_SIGNAL(show_thousands_separator_cb);
-    CONNECT_SIGNAL(show_registers_cb);
-    CONNECT_SIGNAL(accuracy_radio_cb);
-    CONNECT_SIGNAL(accuracy_other_cb);
-    CONNECT_SIGNAL(accuracy_default_cb);    
-    CONNECT_SIGNAL(constant_menu_cb);
-    CONNECT_SIGNAL(function_menu_cb);
-    CONNECT_SIGNAL(store_menu_cb);
-    CONNECT_SIGNAL(recall_menu_cb);
-    CONNECT_SIGNAL(exchange_menu_cb);
-    CONNECT_SIGNAL(mouse_button_cb);
-    /* Detect when populating the right-click menu to enable pasting */
-    CONNECT_SIGNAL(buffer_populate_popup_cb);
-    CONNECT_SIGNAL(shift_cb);
-    CONNECT_SIGNAL(bit_toggle_cb);
-    CONNECT_SIGNAL(aframe_delete_cb);
-    CONNECT_SIGNAL(aframe_activate_cb);
-    CONNECT_SIGNAL(aframe_response_cb);
-    CONNECT_SIGNAL(spframe_delete_cb);
-    CONNECT_SIGNAL(spframe_activate_cb);
-    CONNECT_SIGNAL(spframe_response_cb);
-    CONNECT_SIGNAL(rframe_delete_cb);
-    CONNECT_SIGNAL(rframe_response_cb);
-    CONNECT_SIGNAL(edit_constants_cb);
-    CONNECT_SIGNAL(edit_functions_cb);
-    CONNECT_SIGNAL(edit_constants_delete_cb);    
-    CONNECT_SIGNAL(edit_constants_response_cb);
-    CONNECT_SIGNAL(edit_constants_delete_cb);    
-    CONNECT_SIGNAL(edit_functions_response_cb);
-    CONNECT_SIGNAL(edit_functions_delete_cb);
+    gtk_builder_connect_signals(X.ui, NULL);
 
     X.clipboard_atom = gdk_atom_intern("CLIPBOARD", FALSE);
     X.primary_atom = gdk_atom_intern("PRIMARY", FALSE);
@@ -2730,11 +2610,7 @@ create_kframe()
         g_object_set_data(G_OBJECT(X.disp[i]),
                           "numeric_mode", GINT_TO_POINTER(i));
 
-    /* Put status image into statusbar (glade doesn't support child widgets
-     * in statusbars) */
-    X.status_image = gtk_image_new_from_stock("", GTK_ICON_SIZE_BUTTON);
-    gtk_widget_show(X.status_image);
-    gtk_box_pack_start(GTK_BOX(X.statusbar), X.status_image, FALSE, TRUE, 0);
+    X.status_image = GET_WIDGET("status_image");
 
     /* Set modes for menu items */
     for (i = 1; i < 16; i++) {
@@ -2845,15 +2721,6 @@ ui_load(void)
     if (get_boolean_resource(R_REGS, &boolval))
         ui_set_registers_visible(boolval);
 
-    /* Focus on the clear button */
-    if (X.mode == BASIC) {
-        gtk_window_set_focus(GTK_WINDOW(X.kframe),
-                             GTK_WIDGET(X.clear_buttons[0]));
-    } else {
-        gtk_window_set_focus(GTK_WINDOW(X.kframe),
-                             GTK_WIDGET(X.clear_buttons[1]));
-    }
-
     /* Set default accuracy menu item */
     /* Translators: Accuracy Popup: Menu item to reset the accuracy to the default value. %d is replaced with the default value. */
     SNPRINTF(text, MAXLINE, _("Reset to _Default (%d)"), DEFAULT_ACCURACY);
@@ -2868,6 +2735,13 @@ ui_start(void)
     ui_set_trigonometric_mode(v->ttype);
     ui_set_numeric_mode(v->display.format);
 
+    /* Focus on the clear button */
+    if (X.mode == BASIC) {
+        gtk_widget_grab_focus(GTK_WIDGET(X.clear_buttons[0]));
+    } else {
+        gtk_widget_grab_focus(GTK_WIDGET(X.clear_buttons[1]));
+    }
+    
     gtk_widget_show(X.kframe);
 
     gtk_main();
